-- hds header_start
-- hds header_end
--autori: Martin Proksa
--
--subor: tpg_fun.vhd
--         -funkcny opis generatora testovacich vzoriek (LFSR II)
--         -moznost volby viacerych primitivnych polynomov na zaklade vstup.
--          signalu poly_tpg
--         -moznost volby inicializacneho vektora na zaklade vstup.
--          signalu init_tpg

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;


ENTITY tpg_fun IS
   PORT( 
      clk     : IN     std_logic;
      rst     : IN     std_logic;
      pattern : OUT    std_logic_vector (63 DOWNTO 0)
   );

constant init_tpg : std_logic_vector(63 downto 0) := "0000000000000000000000000000000000000000000000000000000000000001";
constant poly_tpg : std_logic_vector(64 downto 0) := "10000000000000000000000000000000000000000000000000000000000011011";


END tpg_fun ;

-- hds interface_end


architecture tpg_fun_arch of tpg_fun is

	signal lfsr : std_logic_vector(63 downto 0);
	
begin 

process (clk,rst)
	variable polynom: STD_LOGIC_VECTOR (64 downto 0);   
begin
	if (rst = '1') then  --asynchronny reset
		lfsr <= init_tpg;
		polynom := poly_tpg;   
	elsif (clk'event and clk = '1') then
		lfsr(lfsr'length - 1) <= lfsr(0);
		for i in 1 to (lfsr'length - 1) loop
			if polynom(lfsr'length - i) = '1' then
				lfsr(i - 1) <= lfsr(i) xor lfsr(0);
			else
				lfsr(i - 1) <= LFSR(i);
			end if;
		end loop;
	end if;
end process;

   pattern <= lfsr;

end tpg_fun_arch;



