-- hds header_start
-- hds header_end
--autori: Martin Proksa
--        Ivan Varga   
--subor: trc_fun.vhd
--         -funkcny opis pre kompakciu a vygenerovnaie priznaku (LFSR II)
--         -moznost volby viacerych primitivnych polynomov na zaklade vstup.
--          signalu poly_trc
--         -moznost volby inicializacneho vektora na zaklade vstup.
--          signalu init_trc
library IEEE;
use IEEE.std_logic_1164.all;


ENTITY trc_fun IS
   PORT( 
      clk       : IN     std_logic;
      rst       : IN     std_logic;
      trc_in    : IN     std_logic_vector (63 DOWNTO 0);
      signature : OUT    std_logic_vector (63 DOWNTO 0)
   );

constant init_trc : std_logic_vector(63 downto 0) := "0000000000000000000000000000000000000000000000000000000000000001";
constant poly_trc : std_logic_vector(64 downto 0) := "10000000000000000000000000000000000000000000000000000000000011011";

END trc_fun ;

-- hds interface_end


architecture trc_fun_arch of trc_fun is

begin

process (clk,rst)
	-- primitivny polynom
	variable polynom: STD_LOGIC_VECTOR (64 downto 0);
	variable crc: STD_LOGIC_VECTOR (63 downto 0);
	variable last: STD_LOGIC;
begin
	if (rst = '1') then  --asynchronny reset
		polynom := poly_trc;
		crc := init_trc;
		signature <= crc;
	elsif (clk'event and clk = '1') then
		last := crc(63);
		for i in 63 downto 1 loop
			crc(i) := crc(i - 1) xor trc_in(i) xor (polynom(i) and last);
		end loop;
		crc(0) := last xor trc_in(0);
		signature <= crc;
	end if;
end process;

end trc_fun_arch;

 
